<?php

/**
 * @file
 * Definition of Drupal\display\DisplayFormController.
 */

namespace Drupal\display;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityFormController;
use Drupal\layout\Plugin\Core\Entity\Display;

/**
 * Form controller for the display edit forms.
 */
class DisplayFormController extends EntityFormController {

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::form().
   */
  public function form(array $form, array &$form_state, EntityInterface $display) {
    // Get list of layouts to be exposed for display layout selection.
    $layouts = drupal_container()->get('plugin.manager.layout')->getDefinitions();
    $layout_options = array();
    foreach ($layouts as $key => $layout) {
      $layout_options[$key] = $layout['title'];
    }
    $layout_keys = array_keys($layout_options);

    // Add default values to form_state['values'].
    if (!isset($form_state['values'])) {
      $form_state['values'] = array();
    }
    $form_state['values'] += array(
      'layout' => isset($display->layout) ? $display->layout : reset($layout_keys),
    );

    $form['layout'] = array(
      '#type' => 'select',
      '#title' => t('Layout'),
      '#default_value' => $form_state['values']['layout'],
      '#options' => $layout_options,
      '#ajax' => array(
        'callback' => 'display_ajax_block_placement_callback',
        'wrapper' => 'display-blocks',
        'method' => 'replace',
        'effect' => 'fade',
      ),
    );

    // To support the Ajax interaction, remap the display to the newly selected
    // layout. This will reorganize the blocks as appropriate.
    $layout = layout_manager()->createInstance($form_state['values']['layout'], array());
    $display->remapToLayout($layout);

    // Add block editing interface wrapper for Ajax operation.
    $form['blocks'] = array(
      '#prefix' => '<div id="display-blocks">',
      '#suffix' => '</div>',
      'demonstration' => $this->layoutDemonstration($display),
    );

    return parent::form($form, $form_state, $display);
  }

  /**
   * Produces a render array demonstration form of the display.
   */
  private function layoutDemonstration(EntityInterface $display) {

    // Render the layout in an admin context with region demonstrations.
    $layout = layout_manager()->createInstance($display->layout, array());
    $regions = $layout->getRegions();
    foreach ($regions as $region => $info) {
      $region_text = '<div class="region-wrapper"><div class="region_information"><span class="region_label">' . check_plain($info['label']) . '</span></div>';
      $region_text .= '<div class="region_table"><div class="region_blocks connectedSortable">';
      $region_text .= '<div class="add_block"><span class="plus_icon"></span></div></div></div></div>';
      $regions[$region] = $region_text;
    }
    $build['demonstration'] = array(
      '#type' => 'markup',
      '#markup' => $layout->renderLayout(TRUE, $regions),
    );
    $build['#attached']['css'][] = drupal_get_path('module', 'display') . '/display.admin.css';
    $build['#attached']['js'][] = drupal_get_path('module', 'display') . '/display.admin.js';
    $build['#attached']['library'][] = array('system', 'jquery.ui.sortable');
    return $build;
  }

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::actions().
   */
  protected function actions(array $form, array &$form_state) {
    // Only includes a Save action for the entity, no direct Delete button.
    return array(
      'submit' => array(
        '#value' => t('Save'),
        '#validate' => array(
          array($this, 'validate'),
        ),
        '#submit' => array(
          array($this, 'submit'),
          array($this, 'save'),
        ),
      ),
    );
  }

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::save().
   */
  public function save(array $form, array &$form_state) {
    $display = $this->getEntity($form_state);
    $display->save();

    watchdog('display', 'Layout @label saved.', array('@label' => $display->label()), WATCHDOG_NOTICE);
    drupal_set_message(t('Layout %label saved.', array('%label' => $display->label())));

    $form_state['redirect'] = 'admin/structure/layouts';
  }

}

