/**
 * @file toolbar.js
 *
 * Defines the behavior of the Drupal administration toolbar.
 */
(function ($, Drupal, drupalSettings) {

"use strict";

Drupal.toolbar = Drupal.toolbar || {};

/**
 * Store the state of the active tab so it will remain active across page loads.
 */
var activeTab = JSON.parse(localStorage.getItem('Drupal.toolbar.activeTab'));

/**
 * Store the state of the trays to maintain them across page loads.
 */
var orientation = JSON.parse(localStorage.getItem('Drupal.toolbar.trayOrientation')) || 'vertical';
var locked = JSON.parse(localStorage.getItem('Drupal.toolbar.trayLocked')) || false;

/**
 * Holds the jQuery objects of the toolbar DOM element, the trays and messages.
 */
var $toolbar;
var $trays;
var $messages;

/**
 * Holds the mediaQueryList object.
 */
var mql = {
  standard: null,
  wide: null
};

/**
 * Register tabs with the toolbar.
 *
 * The Drupal toolbar allows modules to register top-level tabs. These may point
 * directly to a resource or toggle the visibility of a tray.
 *
 * Modules register tabs with hook_toolbar().
 */
Drupal.behaviors.toolbar = {
  attach: function(context) {
    var options = $.extend(this.options, drupalSettings.toolbar);
    $toolbar = $(context).find('#toolbar').once('toolbar');
    if ($toolbar.length) {
      // Append a messages element for appending interaction updates for screen
      // readers.
      $messages = $(Drupal.theme('toolbarMessageBox')).appendTo($toolbar);
      // Store the trays in a scoped variable.
      $trays = $toolbar.find('.tray');
      $trays
        // Add the tray orientation toggles.
        .find('.lining')
        .append(Drupal.theme('toolbarOrientationToggle'));
      // Set the active state of the orientation toggles.
      changeOrientation(orientation, locked);
      // Store media queries.
      mql.standard = window.matchMedia(options.breakpoints['module.toolbar.standard']);
      // Set up switching between the vertical and horizontal presentation
      // of the toolbar trays based on a breakpoint.
      mql.wide = window.matchMedia(options.breakpoints['module.toolbar.wide']);
      mql.wide.addListener(Drupal.toolbar.mediaQueryChangeHandler);
      if (mql.wide.matches) {
        Drupal.toolbar.mediaQueryChangeHandler(mql.wide);
      }
      // Call setHeight on screen resize. Wrap it in debounce to prevent
      // setHeight from being called too frequently.
      var setHeight = Drupal.debounce(Drupal.toolbar.setHeight, 250);
      // Attach behavior to the window.
      $(window)
        .on('resize.toolbar', setHeight);
      // Attach behaviors to the toolbar.
      $toolbar
        .on('click.toolbar', '.bar a', Drupal.toolbar.toggleTray)
        .on('click.toolbar', '.toggle-orientation button', Drupal.toolbar.orientationChangeHandler);
      // Decorate the main menu with an interactive menu.
      $toolbar.find('.administration.tray .toolbar-menu > .menu').toolbarMenu();
      // Restore the open tab. Only open the tab on wide screens.
      if (activeTab && window.matchMedia(options.breakpoints['module.toolbar.standard']).matches) {
        $toolbar.find('[data-toolbar-tray="' + activeTab + '"]').trigger('click.toolbar');
      }
      // Recalculate the offset top of the toolbar once on initialization.
      else {
        Drupal.toolbar.setHeight();
      }
    }
  },
  // Default options.
  options: {
    breakpoints: {
      'module.toolbar.standard': '',
      'module.toolbar.wide': ''
    }
  }
};

/**
 * Toggle a toolbar tab and the associated tray.
 */
Drupal.toolbar.toggleTray = function (event) {
  var strings = {
    opened: Drupal.t('opened'),
    closed: Drupal.t('closed')
  };
  var $tab = $(event.target);
  var name = $tab.attr('data-toolbar-tray');
  // Activate the selected tab and associated tray.
  var $activateTray = $toolbar.find('[data-toolbar-tray="' + name + '"].tray').toggleClass('active');
  if ($activateTray.length) {
    event.preventDefault();
    event.stopPropagation();
    $tab.toggleClass('active');
    // Toggle aria-pressed.
    var value = $tab.attr('aria-pressed');
    $tab.attr('aria-pressed', (value === 'false') ? 'true' : 'false');
    // Append a message that a tray has been opened.
    $messages.html(
      Drupal.theme('toolbarTrayMessage',
        Drupal.t('@tray tray @state.', {
          '@tray': name,
          '@state': (value === 'true') ? strings.closed : strings.opened
        })
      )
    );
    // Store the active tab name or remove the setting.
    if ($tab.hasClass('active')) {
      localStorage.setItem('Drupal.toolbar.activeTab', JSON.stringify(name));
    }
    else {
      localStorage.removeItem('Drupal.toolbar.activeTab');
    }
    // Disable non-selected tabs and trays.
    $toolbar.find('.bar a')
      .not($tab)
      .removeClass('active')
      // Set aria-pressed to false.
      .attr('aria-pressed', 'false');
    $toolbar.find('.tray').not($activateTray).removeClass('active');
    // Adjust the body to accommodate trays.
    setBodyState();
    // Adjust the height of the toolbar.
    Drupal.toolbar.setHeight();
  }
};

/**
 * The height of the toolbar offsets the top of the page content.
 *
 * Page components can register with the offsettopchange event to know when
 * the height of the toolbar changes.
 */
Drupal.toolbar.setHeight = function () {
  // Set the top of the all the trays to the height of the bar.
  var barHeight = $toolbar.find('.bar').outerHeight();
  var height = barHeight;
  var bhpx =  barHeight + 'px';
  var tray;
  for (var i = 0, il = $trays.length; i < il; i++) {
    tray = $trays[i];
    if (!tray.style.top.length || (tray.style.top !== bhpx)) {
      tray.style.top = bhpx;
    }
  }
  /**
   * Get the height of the active tray and include it in the total
   * height of the toolbar.
   */
  height += $trays.filter('.active.horizontal').outerHeight() || 0;
  // Indicate the height of the toolbar in the attribute data-offset-top.
  var offset = parseInt($toolbar.attr('data-offset-top'), 10);
  if (offset !== height) {
    $toolbar.attr('data-offset-top', height);
    // Alter the padding on the top of the body element.
    $('body').css('padding-top', height);
    $(document).trigger('offsettopchange', height);
    $(window).trigger('resize');
  }
};

/**
 * Respond to configured media query applicability changes.
 */
Drupal.toolbar.mediaQueryChangeHandler = function (mql) {
  if (!locked) {
    var orientation = (mql.matches) ? 'horizontal' : 'vertical';
    changeOrientation(orientation);
    // Adjust the body to accommodate trays.
    setBodyState();
    // Adjust the height of the toolbar.
    Drupal.toolbar.setHeight();
  }
};

/**
 * Respond to the toggling of the tray orientation.
 */
Drupal.toolbar.orientationChangeHandler = function (event) {
  event.preventDefault();
  event.stopPropagation();
  var $button = $(event.target);
  var orientation = event.target.value;
  var $tray = $button.closest('.tray');
  changeOrientation(orientation, true);
  // Adjust the body to accommodate trays.
  setBodyState();
  // Adjust the height of the toolbar.
  Drupal.toolbar.setHeight();
};

/**
 * Change the orientation of the tray between vertical and horizontal.
 *
 * @param {String} newOrientation
 *   Either 'vertical' or 'horizontal'. The orientation to change the tray to.
 *
 * @param {Boolean} isLock
 *   Whether the orientation of the tray should be locked if it is being toggled
 *   to vertical.
 */
function changeOrientation (newOrientation, isLock) {
  var oldOrientation = orientation;
  if (isLock) {
    locked = (newOrientation === 'vertical');
    if (locked) {
      localStorage.setItem('Drupal.toolbar.trayLocked', JSON.stringify(locked));
    }
    else {
      localStorage.removeItem('Drupal.toolbar.trayLocked');
    }
  }
  if ((!locked && newOrientation === 'horizontal') || newOrientation === 'vertical') {
    $trays
      .removeClass('horizontal vertical')
      .addClass(newOrientation);
    orientation = newOrientation;
    toggleOrientationToggle((newOrientation === 'vertical') ? 'horizontal' : 'vertical');
  }
  // Store the orientation to maintain it across page loads.
  localStorage.setItem('Drupal.toolbar.trayOrientation', JSON.stringify(newOrientation));
}

/**
 * Mark up the body tag to reflect the current state of the toolbar.
 */
function setBodyState () {
  var $activeTray = $trays.filter('.active');
  $('body')
    .toggleClass('toolbar-tray-open', !!$activeTray.length)
    .toggleClass('toolbar-vertical', (!!$activeTray.length && orientation === 'vertical'))
    .toggleClass('toolbar-horizontal', (!!$activeTray.length && orientation === 'horizontal'));
}

/**
 * Change the orientation toggle active state.
 */
function toggleOrientationToggle (orientation) {
  $trays.find('.toggle-orientation button').removeClass('active');
  $trays.find('.toggle-orientation button[value="' + orientation + '"]').addClass('active');
}

/**
 * A toggle is an interactive element often bound to a click handler.
 *
 * @return {String}
 *   A string representing a DOM fragment.
 */
Drupal.theme.toolbarOrientationToggle = function () {
  return '<div class="toggle-orientation"><div class="lining">' +
    '<button type="button" value="vertical">' + Drupal.t('Vertical') + '</button>' +
    '<button type="button" value="horizontal">' + Drupal.t('Horizontal') + '</button>' +
    '</div></div>';
};

/**
 * A region to post messages that a screen reading UA will announce.
 *
 * @return {String}
 *   A string representing a DOM fragment.
 */
Drupal.theme.toolbarMessageBox = function () {
  return '<div id="toolbar-messages" class="element-invisible" role="region" aria-live="polite"></div>';
};

/**
 * Wrap a message string in a p tag.
 *
 * @return {String}
 *   A string representing a DOM fragment.
 */
Drupal.theme.toolbarTrayMessage = function (message) {
  return '<p>' + message + '</p>';
};

}(jQuery, Drupal, drupalSettings));
